use strict;
use Test::More tests => 8;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

local $ENV{'EPAGES_TESTSITE'} = "Store" unless $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();

    my $System = LoadRootObject();
    my $Address = $System->insertChild('Address', {
        'Alias'   => 'MyTestAddress',
        'LastName' => 'Tes<<>ter',
        'Street'  => 'Leutrag&ltraben 1',
        'City'    => 'Je&#000060na',
        'Zipcode' => '077&#x3c;43',
        'Company' => 'epag%3e;es',
        'BankAccountHolder' => 'Max &gt;Mustermann',
        'BankCode' => '10020&#0000%3e;062055',
        'FiscalCode' => 'DE123456789<>'
    });

    my $hResult = $Address->get([qw(LastName Street City Zipcode Company BankAccountHolder BankCode FiscalCode)]);

    ok($hResult->{'LastName'} eq 'Tester', 'LastName contains no suspicious characters');
    ok($hResult->{'Street'} eq 'Leutragraben 1', 'Street contains no suspicious characters');
    ok($hResult->{'City'} eq 'Jena', 'City contains no suspicious characters');
    ok($hResult->{'Zipcode'} eq '07743', 'Zipcode contains no suspicious characters');
    ok($hResult->{'Company'} eq 'epages', 'Company contains no suspicious characters');
    ok($hResult->{'BankAccountHolder'} eq 'Max Mustermann', 'BankAccountHolder contains no suspicious characters');
    ok($hResult->{'BankCode'} eq '10020055', 'BankCode contains no suspicious characters');
    ok($hResult->{'FiscalCode'} eq 'DE123456789', 'FiscalCode contains no suspicious characters');

    $Address->delete;

    DisconnectSite();
}

RunScript(Sub=>\&Main, Debug=>1);
