use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");

use Perl::Critic::Policy::DE_EPAGES::BuiltinFunctions::ProhibitForeachQw; ##no critic(RequireCorrectImports)
# Tests Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitForeachQw
#

sub Main {

    my $critic = Perl::Critic->new(
        -theme    => 'epages',
        -severity => 5,
        -profile  => 'NONE',
    );
    ok( defined $critic, 'Perl::Critic initialized' );

    $critic->add_policy( -policy => 'DE_EPAGES::BuiltinFunctions::ProhibitForeachQw' );
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::BuiltinFunctions::ProhibitForeachQw' }
    $critic->policies();
    ok( $policyLoaded, 'Policy loaded' );

    ##no critic qw(ProhibitImplicitNewlines ProhibitMissingImports ProhibitUnusedVariables RequireInterpolationOfMetachars)
    my $sourceCode = q(
        test code
    );
    ok( !$critic->critique( \$sourceCode ), 'funtion not found' );

    $sourceCode = q(
        foreach (qw(test)) {}
    );
    ok( !$critic->critique( \$sourceCode ), 'qw used correctly' );

    $sourceCode = q(
        foreach my $a qw()
    );
    ok( $critic->critique( \$sourceCode ), 'foreach my $a qw used' );

    $sourceCode = q(
        foreach $a qw()
    );
    ok( $critic->critique( \$sourceCode ), 'foreach $a qw used' );

    $sourceCode = q(
        for $a qw()
    );
    ok( $critic->critique( \$sourceCode ), 'for $a qw used' );

    $sourceCode = q(
        foreach my $a qw[] {}
    );
    ok(! $critic->critique( \$sourceCode ), 'different parens in qw used' );

    $sourceCode = q(
        foreach my $a qw ( a b ) {}
    );
    ok( $critic->critique( \$sourceCode ), 'foreach qw ( ) with spaces' );

    $sourceCode = q(
        foreach my
    );
    ok(! $critic->critique( \$sourceCode ), 'testing for NPE' );

    $sourceCode = q(
        print "$_\n" foreach qw( A B C ) ;
    );
    ok(! $critic->critique( \$sourceCode ), 'foreach after statement' );

    done_testing();
}

RunScript( Sub => \&Main );
