/*global define*/

define('de_epages/externalcontent/gadgetdialog/contentslider-view', [
    'jquery',
    'ep',
    './constants',
    './config-view',
    '$tmpl!./contentslider-view',
    '$tmpl!./contentslider-view-list',
    '$dict!../dictionary',
    'de_epages/shop/linkpicker',

    '../ui/contentslider',
    '../ui/cropslidedialog',
    'jquery/ui/sortable',
    'ep/ui/slider',
    'ep/ui/colorpicker',
    'de_epages/mediagallery/ui/filemanagerdialog'
], function ($, ep, CONSTANTS, ConfigView, template, listTemplate, dict, linkpicker) {
    'use strict';

    return ConfigView.extend({

        template: template,

        alias: 'contentslider',

        dict: dict,

        cropDialogOptions: {
            width: 1070,
            minHeight: 570,
            modal: true,
            title: dict.translate('{CropImage}')
        },

        defaultSize: {
            width: 1050,
            height: 350
        },

        events: $.extend({}, ConfigView.events, {
            'colorSelect button': 'onColorSelect',
            'click .ep-sprite.ico_s_delete': 'deleteSlide',
            'click .ep-sprite.object_ico_s_link': 'linkSlide',
            'click .ep-sprite.ico_s_edit': 'editSlide',
            //'slide': 'onSlide',
            'sortstart': 'onSortstart',
            'sortupdate': 'onSortupdate'
        }),

        onColorSelect: function (event, color) {
            this.elements.borderColorpicker.css('background', color);

            if (color === 'transparent') {
                this.elements.borderColorpicker.css({
                    'background-image': 'url(' + ep.config.iconsRoot + '/img_transparent.gif)',
                    'background-size': '100%'
                });
            }

            this.setOption('border', color, true);
        },

        onSlide: function (event, ui) {
            this.setOption('displaytime', ui.value, true);
        },

        onSortstart: function (event, ui) {
            this.sortstartedOnPosition = ui.item.prevAll().length;
        },

        onSortupdate: function (event, ui) {
            var slides = this.gOptions.slides;

            // Reorder *slides*.
            slides.splice(ui.item.prevAll().length, 0, slides.splice(this.sortstartedOnPosition, 1)[0]);
            this.setOption('slides', slides, true);
        },

        deleteSlide: function (event) {
            var $li = $(event.target).parent().parent(),
                slides = this.gOptions.slides;

            this.deletedSlidesStack.push(slides.splice($li.prevAll().length, 1)[0].fullpath);

            $li.remove();

            this.elements.slidesList.sortable('refresh');
            this.setOption('slides', slides, true);
        },

        linkSlide: function (event) {
            var $li = $(event.target).parent().parent(),
                index = $li.prevAll().length;

            linkpicker.open({

                linkText: this.gOptions.slides[index].title || '',
                linkUrl: this.gOptions.slides[index].href || '',
                linkType: this.gOptions.slides[index].linkType || '',
                callback: $.proxy(function (title, url, openNewWindow, linkType) {
                    var slide = this.contentsliderView.gOptions.slides[this.index];

                    slide.href = url;
                    slide.title = title;
                    slide.target = openNewWindow ? '_blank' : '';
                    slide.linkType = linkType;
                    this.contentsliderView.setOption('slides', this.contentsliderView.gOptions.slides, true);
                }, {
                    contentsliderView: this,
                    index: index
                })
            });
        },

        editSlide: function (event) {
            var $li = $(event.target).parent().parent(),
                index = $li.prevAll().length;

            this.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex', index);
            this.elements.addImage.mediagalleryUiFilemanagerdialog('open');
        },

        renderPreview: function () {
            var self = this,
                preview = self.elements.preview,
                parent = preview.parent(),
                gadgetPath = CONSTANTS.gWidgetNamespace + self.alias.charAt(0).toUpperCase() + self.alias.slice(1);

            preview.remove();

            preview = self.elements.preview = $('<div>').insertBefore(self.elements.footer);

            preview[gadgetPath]($.extend(true, {}, self.gOptions, {
                cycleOptions: {
                    after: $.proxy(function (currSlideElement, nextSlideElement, options, forwardFlag) {
                        this.showLinkInfo($(nextSlideElement).prevAll().length);
                    }, this)
                }
            }));

            preview.find('a[href]').each(function () {
                $(this).removeAttr('href');
            }); // Links should not work inside the dialog.

            if (self.gOptions.slides.length === 1) {
                self.showLinkInfo(0);
            }

            if (!self.gOptions.slides.length) {
                self.elements.linkInfo.html('');
            }
        },

        showLinkInfo: function (index) {
            var slide = this.gOptions.slides[index];

            if (slide.title || slide.href) {
                this.elements.linkInfo.html(dict.translate('{LinkedWith}') + ': <b>' + slide.title || slide.href + '</b>');
            } else {
                this.elements.linkInfo.html('');
            }
        },

        render: function () {
            var self = this,
                renderedTemplate = self.template(self.gOptions);
            self.elements = renderedTemplate.tmplItem('elements');

            // Init *displaytimeSlider*.
            self.elements.displaytimeSlider.uiSlider({
                slide: $.proxy(this.onSlide, this)
            });

            // Translate text nodes.
            renderedTemplate
                .dictParse(dict, true)
                .find(':input, .ep-sprite')
                .dictParseAttr(dict, ['placeholder', 'title'])
                .end()
                .find(':input')
                .uiInput();

            self.$el.append(renderedTemplate);

            // Init *sortable* on *slidesList*.
            $(self.elements.slidesList).sortable({
                cursor: 'move'
            });

            // Init *borderColorpicker*.
            $(self.elements.borderColorpicker).uiColorpicker({
                color: self.gOptions.border
            });

            // Init *addImage* button.
            self.elements.addImage = self.elements.addImage.mediagalleryUiFilemanagerdialog({
                contentsliderSlideIndex: null,
                filemanager: {
                    selectable: 'image/'
                },
                //#JSCOVERAGE_IF 0
                dialog: {
                    close: $.proxy(function (event, ui) {
                        self.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex', null);
                    }, self),
                    buttons: [
                        {
                            text: dict.translate('{Apply}'),
                            click: $.proxy(function () {
                                var selectedFiles = self.elements.addImage.mediagalleryUiFilemanagerdialog('getSelectedElements');
                                if (selectedFiles.length) {
                                    self.cropImage(selectedFiles[0], self.elements.addImage.mediagalleryUiFilemanagerdialog('option', 'contentsliderSlideIndex'));
                                }
                                self.elements.addImage.mediagalleryUiFilemanagerdialog('close');
                            }, self)
                        }
                    ]
                }
            });

            self.renderPreview();

            return this;
        },

        //#JSCOVERAGE_ENDIF
        initialize: function () {
            this.gDefaults = $.extend(true, {}, CONSTANTS.gDefaults[this.alias]);
            this.plugin = CONSTANTS.gNamespace + '.ui.' + this.alias;
            this.gOptions = $.isEmptyObject(this.options) ? this.gDefaults : this.options;
            this.deletedSlidesStack = [];
        },
        //#JSCOVERAGE_IF 0

        addImage: function (name, fullpath, index) {
            if (typeof index !== 'undefined' && index !== null) {
                this.deletedSlidesStack.push(this.gOptions.slides[index].fullpath);

                this.gOptions.slides[index].name = name;
                this.gOptions.slides[index].fullpath = fullpath;


                $(this.elements.slidesList.children('li').get(index)).find('.ep-fileName').text(name);
                this.setOption('slides', this.gOptions.slides, true);
            } else {
                var listView = listTemplate({
                    name: name,
                    fullpath: fullpath
                });

                // Translation.
                listView.find('.ep-sprite[title]').each(function () {
                    this.title = dict.translate(this.title);
                });

                this.elements.slidesList.append(listView).sortable('refresh');
                this.gOptions.slides.push({
                    name: name,
                    fullpath: fullpath
                });
                this.setOption('slides', this.gOptions.slides, true);
            }
        },

        cropImage: function (file, index) {
            var self = this,
                fullpath = file.get('fullpath'),
                height = file.get('height'),
                width = file.get('width'),
                setting = 'toosmallxy';

            // If any of these three attributes are falsy, we lay back and do nothing.
            if (!(fullpath && height && width)) {
                return;
            }

            if (height === self.defaultSize.height.toString() && width === self.defaultSize.width.toString()) {
                ep.ajax({
                    type: 'POST',
                    dataType: 'json',
                    data: {
                        ViewAction: 'JSONViewResponse',
                        ChangeAction: 'JSONUploadContentSliderImage',
                        ObjectID: ep.config.siteId,
                        Path: fullpath
                    }
                }).done(function (data) {
                    if (data.Errors) {
                        return;
                    }
                    self.addImage(data.filename, data.fullpath, index);
                });
                return;
            }
            self.cropImageSetting = setting;

            self.cropImageDialog = $('<div>').externalcontentUiCropslidedialog({
                url: fullpath,
                dialogOptions: $.extend({}, self.cropDialogOptions, {
                    close: function (event, ui) {
                        self.cropImageDialog.externalcontentUiCropslidedialog('destroy');
                    },
                    buttons: [
                        {
                            text: dict.translate('{Apply}'),
                            click: $.proxy(function () {
                                ep.ajax({
                                    type: 'POST',
                                    dataType: 'json',
                                    data: {
                                        ViewAction: 'JSONViewResponse',
                                        ChangeAction: 'JSONUploadContentSliderImage',
                                        ObjectID: ep.config.siteId,
                                        Path: fullpath,
                                        DataURL: self.cropImageDialog.externalcontentUiCropslidedialog('getFabricCanvas').toDataURL()
                                    }
                                }).done(function (data) {
                                    if (data.Errors) {
                                        return;
                                    }
                                    self.addImage(data.filename, data.fullpath, index);
                                });
                                $(self.cropImageDialog).uiDialog('close');

                            }, self)
                        }
                    ]
                })
            });
        }
    });
});