/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep
 *
 * @revision		$Revision: 1.4.2.1 $
 */

define("ep", [
	"module",
	"jquery",
	"util/string",

	"jquery/each",
	"jquery/mixin",
	"jquery/event/fix",
	"ep/modify"
], function (mod, $, str) {

	// Simulate subclass of jQuery
	var rootjQuery = $(document),
		ep = function ( selector, context ) {
			return new $.fn.init( selector, context, rootjQuery );
		};
	// Get all properties of jQuery to ep
	$.extend( true, ep, $ );
	// Setup prototype context
	ep.fn = $.fn = $.prototype;

	// Merge config params
	ep.config = $.extend({
		debugLevel:					0,
		version:					'',
		webRoot:					'',
		storeWebRoot:				'',
		storeRoot:					'',
		storeTypeRoot:				'',
		siteRoot:					'',
		siteId:						'0',
		baseUrl:					undefined,
		iconsRoot:					'',
		objectId:					'0',
		viewAction:					'',
		storeFrontUrl:				'',
		requestProtocolAndServer:	'',
		maxContentLength:			'',
		currency:					'',
		autoMetaparse:				true
	}, mod.config());

	var splice = Array.prototype.splice,
		push = Array.prototype.push,
		extend = $.extend,
		mixin = $.mixin,
		ready = $.fn.ready,
		browser = $.browser,
		browserClass = (/khtml/i).test(navigator.userAgent) ? 'khtml' : '';

	// add browser engine name as class to html tag
	$.each( ['mozilla', 'msie', 'webkit', 'opera'], function( i, name ){
		if( browser[name] ){
			browserClass = name;
			return false;
		}
	});
	$('html').addClass( browserClass );

	$.extend(ep,{
		// similar to $.extend; if only 1 hash in arguments ep will extend with this hash
		extend: function() {
			return extend.apply( this, arguments );
		},
		// similar to $.mixin; if only 1 hash in arguments ep will mixin with this hash
		mixin: function() {
			return mixin.apply( this, arguments );
		}
	});

	ep.extend({
		// Warn handle for deprecated functionality
		deprecated: function (msg, trace) {
			if (typeof console !== "undefined") {
				if (console.warn) {
					console.warn("EP deprecated: " + msg);
				}
				if (trace && console.trace) {
					console.trace();
				}
			}
		},

		// find a key by the givn value in a hash
		inHash: function( val, obj ){
			var ret;
			if( !$.isPlainObject(obj) ){
				return ret;
			}
			$.each( obj, function( key, value ){
				if( value === val ){
					ret = key;
					return false;
				}
			});
			return ret;
		},

		// return array of keys, values or keys and values from hash
		hashToArray: function( obj, type ){
			var keys = [],
				values = [];
			$.each( obj, function( key, value ){
				keys.push( key );
				values.push( value );
			});
			return (/^k/i).test(type) ? keys : ( (/^v/i).test(type) ? values : [keys,values] );
		},

		// build a hash from 2 arrays (keys values)
		arrayToHash: function( keys, values ){
			if( !values ){
				values = keys;
				keys = [];
			}

			var hash = {};
			$.each( values, function( i, elm ){
				hash[ keys[i] || i ] = elm;
			});
			return hash;
		},

		// add item(s) to an array or array-like object at an indexed position without overwrite existing items
		insert: function( target, insert, index ){
			ep.deprecated("use native array method '.unsplice()' instead of 'ep.insert()'", true);

			index = typeof index !== 'number' ? target.length : index;

			var end = splice.call( target, index, target.length-index );

			if( $.isArray(insert) ){
				push.apply( target, insert );
			}
			else if( typeof insert === 'object' && typeof insert.length === 'number' ){
				$.merge( target, insert );
			}
			else{
				push.call( target, insert );
			}

			return $.merge( target, end );
		},

		// escape string to use it for RegExp
		escapeExpStr: function( string, except ){
			ep.deprecated("'ep.escapeExpStr()' is replaced by '.escExpStr()' in module 'util/string'", true);

			return str.escExpStr(string, except);
		},

		// scrunch a string [ fulltext... , full...text , ...fulltext ]
		scrunch: function( string, length, options ) {
			ep.deprecated("'ep.scrunch()' is replaced by '.shrink()' in module 'util/string'", true);

			if( typeof length === 'object' ){
				options = length;
				length = null;
			}
			
			if (!options) {
				options = {};
			}

			if (length) {
				options.length = length;
			}

			return str.shrink(string, options);
		},

		// reverse a string or number or array
		reverse: function( string ) {
			if( $.isArray(string) ){
				ep.deprecated("use native array method '.reverse()' instead of 'ep.reverse()'", true);

				return string.reverse();
			}
			else {
				ep.deprecated("'ep.reverse()' is replaced by '.reverse()' in module 'util/string'", true);

				return str.reverse(string);
			}
		},

		// remove xml/xhtml/html tags
		stripTags: function( string ) {
			ep.deprecated("'ep.stripTags()' is replaced by '.stripTags()' in module 'util/string'", true);

			return str.stripTags(string);
		},

		// hash to address functions as global object in inline scripts
		local: {

		}
	});

	ep.fn.setDefault = function(value){
		this.each( function(i, element){
			var $element = $(element),
					tagName = element.tagName.toLowerCase();

			if(tagName === 'input' || tagName === 'textarea'){
				if( element.type === 'radio' || element.type === 'checkbox' ){
					$element
						.attr('checked', value)
						.prop('defaultChecked', value);
				}
				else{
					$element
						.val(value)
						.prop('defaultValue', value);
				}
			}
			else if(tagName === 'select'){
				element.options[element.selectedIndex].defaultSelected = false;
				element.value = value;
				element.options[element.selectedIndex].defaultSelected = true;
			}
		});

		return this;
	};

	// Expose ep to the global object
	window.ep = ep;

	return ep;
});