define(['testsuite/qunit', 'jquery', 'ep/ui/richtable', '$ready!'], function (QUnit, $, ep) {

	QUnit.module("ep/ui/richtable");

	QUnit.asyncTest("ep/ui/richtable", function() {

		// create content lists
		var richtable = $('#contentTable')
				.uiRichtable({
					'region': jQuery.i18n.settings.region,
					'currency': jQuery.i18n.settings.currency,
					'editIcon': true,
					'alterIcon': true
				}),
				richtable2 = $('#contentTable2')
				.uiRichtable({
					'region': jQuery.i18n.settings.region,
					'currency': jQuery.i18n.settings.currency,
					'editIcon': false
				});

		QUnit.strictEqual(
			richtable.uiRichtable('Instance') instanceof $.Widget,
			true,
			"Richtable widget is created and instance of jQuery.Widget"
		);

		$.each(['_create', '_handleEditable', '_setContent'], function (i, fn) {

			QUnit.strictEqual(
				$.isFunction($.ui.uiRichtable.prototype[fn]),
				true,
				'$.isFunction(' + fn + '):'
			);

		});

		$('#contentTable td').trigger('mouseover');

		QUnit.strictEqual(
			$('#contentTable td[data-ep-editable] .epEditWrapper').length,
			6,
			"$('#contentTable td[data-ep-editable] .epEditWrapper').length"
		);

		QUnit.strictEqual(
			$('#contentTable td[data-ep-editable] .epEditWrapper i').length,
			6,
			"amount of edit icons in 1st table"
		);

		// simulate click in first cell
		var event = $.Event('click');
		event.originalTarget = $('#contentTable .OneQuarter:eq(0)')[0];
		$('#contentTable .OneQuarter:eq(0)').trigger(event);

		QUnit.strictEqual(
			$('#contentTable .OneQuarter:eq(0) input').length,
			1,
			"$('#contentTable .OneQuarter:eq(0) input').length"
		);

		event = $.Event('keyup');
		event.keyCode = 27;

		$('#contentTable .OneQuarter:eq(0) input').trigger(event);

		// simulate click in second cell
		event = $.Event('click');
		event.originalTarget = $('#contentTable .OneQuarter:eq(1)')[0];
		$('#contentTable .OneQuarter:eq(1)').trigger(event);

		QUnit.strictEqual(
			$('#contentTable .OneQuarter:eq(1) input').length,
			1,
			"$('#contentTable .OneQuarter:eq(1) input').length"
		);

		event.originalTarget = $('#contentTable .OneQuarter:eq(2)')[0];
		$('.OneQuarter:eq(2)').trigger(event);

		// simulate blurring the input
		$('#contentTable td[data-ep-editable]:first input').first().trigger('blur');
		equal(
			$('#contentTable td[data-ep-editable]:first input').first().is(':hidden'),
			true,
			"$('#contentTable td[data-ep-editable]:first input').first().is(':hidden')"
		);

		// trigger click in Name cell and check if an input was created
		objHelp = $('#contentTable tbody td *[data-ep-attribute=Name]:first').closest('td');
		objHelp.trigger('click');
		equal(
			objHelp.find('input:text').length,
			1,
			".find('input:text').length (Name)"
		);

		// trigger click in StockLevel cell and check if an input was created
		objHelp = $('#contentTable tbody td *[data-ep-attribute=StockLevel]:first').closest('td');
		objHelp.trigger('click');
		equal(
			objHelp.find('input:text').length,
			1,
			".find('input:text').length (Stocklevel)"
		);

		// trigger click in Price cell and check if an input was created
		objHelp = $('#contentTable tbody td *[data-ep-attribute=Price]:first').closest('td');
		objHelp.trigger('click');
		equal(
			objHelp.find('input:text').length,
			1,
			".find('input:text').length (Price)"
		);

		// define enter button event and trigger it
		var e = jQuery.Event('keypress', {keyCode: 13});
		objHelp.find('input:text').trigger(e);
		window.setTimeout(function(){
			equal(
				objHelp.find('input:focus').length,
				0,
				".find('input:focus').length (Price)"
			);

			// start test with delay
			start();
		}, 500);

		// uncheck all checkboxes
		$('#contentTable td.Checkbox input:checkbox').attr('checked', false);

		// trigger click in first checkbox
		objHelp = $('#contentTable tbody td.Checkbox:first input:checkbox');
		objHelp.trigger('click');
		equal(
			objHelp.attr('checked'),
			'checked',
			"checkbox is checked"
		);

		objHelp = $('#contentTable thead td.Checkbox:first input:checkbox').trigger('click');
		equal(
			objHelp.attr('checked'),
			'checked',
			"checkbox is checked"
		);

		equal(
			$('#contentTable tr').hasClass('RowSelected'),
			true,
			".hasClass('RowSelected')"
		);
	});
});