use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::MultiStoreCommon::API::Table::Object;
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $Root = LoadObjectByPath($ROOTPATH);
                my $Test22 = $Root->path('Test-2-2');
                my $Test2211 = $Root->path('Test-2-2/Test-2-2-1/Test-2-2-1-1');
                my $Test3 = $Root->path('"Test 3"');
                my $Test32 = $Test3->path('"Test / ""&amp;"""');

                my $hPathInfo = DE_EPAGES::MultiStoreCommon::API::Table::Object::_GetObjectPathByObjectID({ ## no critic qw( ProtectPrivateSubs )
                        'TableName'     => 'object',
                        'ObjectIdField' => 'objectid',
                        'SiteIdField'   => 'siteid'
                });

                ok( $hPathInfo->{$Test22->id} eq 'Test/Test-2-2', 'Path from Root: ' . $hPathInfo->{$Test22->id} );
                ok( $hPathInfo->{$Test2211->id} eq 'Test/Test-2-2/Test-2-2-1/Test-2-2-1-1',
                    'Path from Object: ' . $hPathInfo->{$Test3->id}
                );
                ok( $hPathInfo->{$Test3->id} eq 'Test/Test_0020_3', 'Path with spaces: ' . $hPathInfo->{$Test22->id} . ' + '. $hPathInfo->{$Test3->id});
                ok( $hPathInfo->{$Test32->id} eq 'Test/Test_0020_3/Test_0020__002F__0020__0022__0026_amp;_0022_',
                    'Path with special characters: ' . $hPathInfo->{$Test32->id}
                );

            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $TestRoot = LoadRootObject()->insertChild('Object',{'Alias'=>'Test'});
    $TestRoot->insertChild('Object',{'Alias'=>'Test-2-1'});
    $TestRoot->insertChild('Object',{'Alias'=>'Test-2-2'})
             ->insertChild('Object',{'Alias'=>'Test-2-2-1'})
             ->insertChild('Object',{'Alias'=>'Test-2-2-1-1'});
    $TestRoot->insertChild('Object',{'Alias'=>'Test 3'})
             ->insertChild('Object',{'Alias'=>'Test / "&amp;"'});

    return 1;
}

RunScript( Sub => \&Main );

1;
