#========================================================================================
#   Requier DemoShop, PageTypes and more from Store
#========================================================================================
use strict;
use Test::More;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadRootObject LoadObjectByPath );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Object::TextProcessor;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError);

# GLOBAL
my $PAGE_TYPE = undef;

sub TestSimple {
    my $Root           = LoadRootObject();
    my $Processor      = DE_EPAGES::Presentation::API::Object::TextProcessor->new();
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PAGE_TYPE, $Root );
    isa_ok( $Processor, 'DE_EPAGES::Presentation::API::Object::TextProcessor' );

    my $hTLEHash = { Var => 'Value' };

    my $Renderd = $Processor->processText( $ObjectPageType, $Root, \'#Var', $hTLEHash, undef );
    is( $$Renderd, 'Value', 'processText - simple TLE processing' );
    eval {$Renderd = $Processor->processText( $ObjectPageType, $Root, '#Var', $hTLEHash, undef );};
    ok(ExistsError && GetError->message =~  m/Parameter Template.+/, 'processText - pass template as string');
}

sub TestLanguage {
    my $Shop           = LoadObjectByPath('/Shops/DemoShop');
    my $Processor      = DE_EPAGES::Presentation::API::Object::TextProcessor->new();
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PAGE_TYPE, $Shop );
    my $aLanguages = $Shop->get('Languages');

    foreach (@$aLanguages) {
        my $Renderd
        = $Processor->processText( $ObjectPageType, $Shop, \'#Shop.Slogan', $Shop->tleHash, $_->{LanguageID} );
        is( $$Renderd, $Shop->get( 'Slogan', $_->{LanguageID} ), 'processText - language context ' . $_->{English} );
    }

}

sub TestWithShopAddress {
    my $Shop           = LoadObjectByPath('/Shops/DemoShop');
    my $Processor      = DE_EPAGES::Presentation::API::Object::TextProcessor->new();
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PAGE_TYPE, $Shop );

    my $Address = $Shop->get('Address');
    my $hOldAddressValues = $Address->get(['FirstName','MiddleName','LastName','DisplayName','Street','City','Zipcode','Company']);

    my $hValues = {
        'FirstName' => 'Max',
        'MiddleName' => undef,
        'LastName' => 'Mustermann',
        'DisplayName' => 'Max van Mustermann',
        'Street' => 'Leutragraben 1',
        'City' => 'Jena',
        'Zipcode' => '07743',
        'Company' => 'ePages',
    };
    $Address->set($hValues);

    my $Text = '<div>#WITH(#Shop.Address)#LocaleInclude("Address",#Country.Code2)#ENDWITH</div>';
    my $result = $Processor->processText( $ObjectPageType, $Shop, \$Text, { 'Shop' => $Shop->tleHash() }, 2);

    $$result =~ s/\n//sg; # Delete new lines;
    $$result =~ s/\s//sg; # Delete whitepaces
    $$result =~ s/<!--(.*?)-->//sg; # Delete HTML comments.

    # Restore old address values.
    $Address->set($hOldAddressValues);

    ## no critic (RegularExpressions::ProhibitComplexRegexes)
    is($$result, '<div><spanclass="vcard"><spanclass="org">ePages</span><br/><spanclass="nfn"><spanclass="given-name">Max</span><spanclass="family-name">Mustermann</span></span><br/><spanclass="adr"><spanclass="street-address">Leutragraben1</span><br/><spanclass="postal-code">07743</span><spanclass="locality">Jena</span><br/></span></span></div>', 'processText - formated address from DemoShop');
    ## use critics
}

#========================================================================================
#   MAIN
#========================================================================================
RunScript(
    Sub => sub {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                my $NumTests = 6;
                plan tests => $NumTests;

                my $ObjectPath = '/Shops/DemoShop';
                if( ExistsObjectByPath( $ObjectPath ) ) {
                    $PAGE_TYPE = LoadPageTypeByAlias('BasePageType');
                }
                SKIP: {
                    skip 'No DemoShop installed.', $NumTests unless defined $PAGE_TYPE;
                    TestSimple();
                    TestLanguage();
                    TestWithShopAddress();
                }
            },
        );
    }
);
