use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
ExistsObjectByPath
LoadObjectByPath
LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Tax::API::CSV::TaxExportImport;
use File::Basename;
use Test::More;

my $ROOTPATH = '/Test';

sub Main {
    ok( Init(), 'initialization' );
    eval {
        my $Shop      = LoadObjectByPath($ROOTPATH)->child('TestShop');
        my $Driver    = DE_EPAGES::Tax::API::CSV::TaxExportImport->new();
        my $TaxMatrix = $Shop->get('TaxMatrix');
        if ( $TaxMatrix->existsChild('Germany') ) {
            my $TaxArea = $TaxMatrix->child('Germany');
            $TaxArea->set( { 'Name' => 'just make sure this is different...' }, GetPKeyLanguageByCode('de') );
        }
        $Driver->csvImport(
            $Shop,
            'importTax.csv', {
                'sep_char'       => ',',
                'LanguageCode2s' => [ 'de', 'en' ],
                'LocaleID'       => 'de_DE',
                'encoding'       => 'iso-8859-1',
                'Monitor'        => undef,
            }
        );
        $TaxMatrix = $Shop->get('TaxMatrix');
        my $TaxArea  = $TaxMatrix->child('Germany');
        my $TaxClass = $TaxMatrix->child('normal');
        is( $TaxArea->get( 'Name', GetPKeyLanguageByCode('de') ), 'Deutschland 2', 'CSV import: localized strings ok' );
        cmp_ok( $TaxMatrix->taxRate( $TaxArea, $TaxClass ), '==', 0.21, '21 percent tax ok' );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( $Error->longMessage );
    }
    ok( Cleanup(), 'cleanup' );
    done_testing();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { 'Alias' => 'Test' } );
    ImportObjects( 'importTax.xml', $ROOTPATH );
    return 1;
}

RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => \&Main,
                );
            }
        );
    },
    Message => 'long',
);
